module C where

data WithCounter a = WC (Int -> (a,Int))

instance Monad WithCounter where
  return x = WC (\i -> (x,i))
  WC f >>= g = WC (\i -> let (v,j) = f i in let WC gg = g v in gg j)

-- Haskell wants monads to also be applicative functors.
-- You can ignore this.
-- {{{
-- It's always possible to obtain applicative functors with
-- these definitions in terms of bind and return.
instance Functor WithCounter where
  fmap f m = m >>= \x -> return (f x)
instance Applicative WithCounter where
  pure = return
  m1 <*> m2 = m1 >>= (\x1 -> m2 >>= (\x2 -> return (x1 x2)))
-- }}}

-- Operations

incr :: WithCounter ()
incr = WC (\i -> ((),i+1))

get :: WithCounter Int
get = WC (\i -> (i,i))

set :: Int -> WithCounter ()
set v = WC (\_ -> ((),v))

run :: WithCounter a -> a
run (WC f) = let (a,_) = f 0 in a

-- Test

myprog :: WithCounter Int
myprog = do x <- get; set 2; set x; incr; get
